import pytest
import aiohttp
from bs4 import BeautifulSoup
from unittest.mock import Mock, patch, AsyncMock
from seo_analyzer.crawler import WebsiteCrawler

@pytest.fixture
def crawler():
    return WebsiteCrawler('https://example.com', max_pages=2, delay=0.1)

@pytest.fixture
def sample_html():
    return """
    <html>
        <head>
            <title>Test Page</title>
        </head>
        <body>
            <a href="https://example.com/page1">Internal Link 1</a>
            <a href="https://example.com/page2">Internal Link 2</a>
            <a href="https://external.com">External Link</a>
            <a href="/relative-path">Relative Link</a>
        </body>
    </html>
    """

@pytest.fixture
def soup(sample_html):
    return BeautifulSoup(sample_html, 'html.parser')

def test_extract_internal_links(crawler, soup):
    internal_links = crawler._extract_internal_links(soup, 'https://example.com')
    assert len(internal_links) == 3  # Two direct internal links and one relative link
    assert 'https://example.com/page1' in internal_links
    assert 'https://example.com/page2' in internal_links
    assert 'https://example.com/relative-path' in internal_links

@pytest.mark.asyncio
async def test_get_page_content(crawler):
    mock_response = AsyncMock()
    mock_response.text.return_value = '<html><body>Test</body></html>'

    mock_session = AsyncMock()
    mock_session.get.return_value.__aenter__.return_value = mock_response
    crawler.session = mock_session

    content, soup = await crawler._get_page_content('https://example.com')
    assert isinstance(soup, BeautifulSoup)
    assert mock_session.get.called
    mock_session.get.assert_called_with(
        'https://example.com',
        headers={'User-Agent': crawler.ua.random}
    )

@pytest.mark.asyncio
async def test_crawl(crawler):
    # Mock the session and response
    mock_response = AsyncMock()
    mock_response.text.return_value = '<html><body><a href="/page1">Link</a></body></html>'

    mock_session = AsyncMock()
    mock_session.get.return_value.__aenter__.return_value = mock_response
    crawler.session = mock_session

    # Mock the analyzer
    mock_analyzer = Mock()
    mock_analyzer.analyze_page.return_value = {'url': 'https://example.com', 'status': 'success'}
    crawler.analyzer = mock_analyzer

    results = await crawler.crawl()
    
    assert isinstance(results, dict)
    assert 'base_url' in results
    assert 'total_pages_crawled' in results
    assert 'crawl_stats' in results
    assert 'page_results' in results
    assert results['base_url'] == 'https://example.com'
    assert results['total_pages_crawled'] > 0

def test_generate_summary(crawler):
    # Setup test data
    crawler.visited_urls = {'https://example.com', 'https://example.com/page1'}
    crawler.results = {
        'https://example.com': {'url': 'https://example.com', 'status': 'success'},
        'https://example.com/page1': {'url': 'https://example.com/page1', 'error': 'Not found'}
    }

    summary = crawler._generate_summary()
    
    assert isinstance(summary, dict)
    assert summary['total_pages_crawled'] == 2
    assert summary['pages_with_errors'] == 1
    assert summary['crawl_stats']['successful_pages'] == 1
    assert summary['crawl_stats']['failed_pages'] == 1